# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from collections import namedtuple, OrderedDict
from datetime import datetime, timedelta, date, time
from decimal import Decimal
import json
import logging
from threading import RLock
from uuid import UUID
from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.validators import validate_slug, validate_ipv46_address, \
    URLValidator, validate_email
from django.db.models import QuerySet, Model
from django import forms
from django.utils.encoding import force_text
from django.utils.encoding import python_2_unicode_compatible
from django.utils.module_loading import import_string
from django.utils.six import string_types, integer_types
from .validators import validate_setting_name, validate_default
from .validators import validate_formish
from django.core.serializers.json import DjangoJSONEncoder


logger = logging.getLogger(__name__)


class JSONEncoder(DjangoJSONEncoder):
    def default(self, o):
        if isinstance(o, datetime):
            # We don't use `T` as the sep, because Django doesn't include it
            # in it's DATETIME_INPUT_FORMATS
            r = o.isoformat(sep=str(' '))
            # We don't strip microseconds, because loss of precision is dumb
            # and I don't actually *care* about ECMA-262 or whatever.

            # We remove the timezone stuff because it ain't in
            # the DATETIME_INPUT_FORMATS
            if r.endswith('+00:00'):
                r = r[:-6]
            return r
        elif isinstance(o, timedelta):
            return force_text(o.total_seconds())
        elif isinstance(o, UUID):
            return force_text(o)
        elif isinstance(o, QuerySet):  # MultipleModelChoice
            return tuple(force_text(x.pk) for x in o)
        elif isinstance(o, Model):  # ModelChoice
            return force_text(o.pk)
        else:
            return super(JSONEncoder, self).default(o)  # pragma: no cover


class RegistryError(KeyError):
    pass

class AlreadyRegistered(RegistryError):
    pass

class NotRegistered(RegistryError):
    pass


Unregistered = namedtuple('Unregistered', 'setting_name form_class default')


@python_2_unicode_compatible
class FormRegistry(object):
    __slots__ = ('_registry', '_defaults', '_name', '_lock')

    def __init__(self, name=None):
        self._registry = {}
        self._defaults = {}
        self._name = name or 'default'
        self._lock = RLock()

    def __str__(self):
        return ', '.join(self._registry.keys())

    def __repr__(self):
        return '<%(mod)s.%(cls)s "%(name)s" [%(settings)s]>' % {
            'mod': self.__module__, 'cls': self.__class__.__name__,
            'name': self._name, 'settings': force_text(self),
        }

    def __len__(self):
        return len(self._registry)

    def ready(self, sender, instance, model):
        project_setting = getattr(settings, 'STAGESETTINGS', {})

        for setting_name, config in project_setting.items():
            default = None
            # if the only parameter is a dictionary, make it a form.
            try:
                validate_default(config)
                form = generate_form(config)
                default = config
            except ValidationError:
                # if the first parameter is a dictionary, generate a form
                try:
                    validate_default(config[0])
                    form = generate_form(config[0])
                    default = config[0]
                # if it's not a dictionary, it's a path to a Form class
                except ValidationError:
                    form = import_string(config[0])
            # if a second parameter is given, it is always the defaults.
            try:
                default = config[1]
            except IndexError:
                pass
            self.register(key=setting_name, form_class=form, default=default)

        db_for_rw = model.objects.using(self._name)
        wanted = set(self.keys())
        existing = set(db_for_rw.filter(key__in=wanted)
                       .values_list('key', flat=True))
        missing = wanted - existing
        models = [model(key=setting_name,
                        raw_value=self.get_default(key=setting_name))
                  for setting_name in missing]
        return db_for_rw.bulk_create(models)

    def register(self, key, form_class, default=None):
        validate_setting_name(key)
        validate_formish(form_class)
        if default is not None:
            validate_default(default)
        with self._lock:
            if key in self._registry:
                raise AlreadyRegistered('The setting "%s" is already registered' % key)
            self._registry[key] = form_class
            self._defaults[key] = default or {}
            return True
    add = register
    __setitem__ = register

    def unregister(self, key):
        validate_setting_name(key)
        with self._lock:
            if key not in self._registry:
                raise NotRegistered('The setting "%s" is not registered' % key)
            existing_form = self._registry.pop(key)
            existing_default = self._defaults.pop(key)
            return Unregistered(setting_name=key, form_class=existing_form,
                                default=existing_default)
    remove = unregister
    pop = unregister
    __delitem__ = unregister

    def __getitem__(self, item):
        return self._registry[item]

    def __nonzero__(self):
        return len(self) > 0
    __bool__ = __nonzero__

    def keys(self):
        return self._registry.keys()

    def values(self):
        return self._registry.values()

    def _get_default(self, key):
        validate_setting_name(key)
        return self._defaults[key]

    def get_default(self, key):
        return self.serialize(self._get_default(key=key))

    def serialize(self, data):
        return json.dumps(data, cls=JSONEncoder)

    def deserialize(self, data):
        return json.loads(data)

registry = FormRegistry(name='default')


def _select_field(v):
    if v is None:
        return forms.NullBooleanField(initial=v)
    elif isinstance(v, datetime):
        return forms.DateTimeField(initial=v)
    elif isinstance(v, date):
        return forms.DateField(initial=v)
    elif isinstance(v, time):
        return forms.TimeField(initial=v)
    elif isinstance(v, timedelta):
        return forms.DurationField(initial=v)
    elif isinstance(v, Decimal):
        return forms.DecimalField(initial=v)
    elif isinstance(v, float):
        return forms.FloatField(initial=v)
    elif isinstance(v, bool):
        return forms.BooleanField(initial=v, required=False)
    elif isinstance(v, integer_types):
        return forms.IntegerField(initial=v)
    elif isinstance(v, UUID):
        try:
            return forms.UUIDField(initial=v)
        except AttributeError:
            logger.error("You're on an older version of Django which doesn't "
                         "have UUIDField, so a charfield is being used "
                         "instead", exc_info=1)
            return forms.CharField(initial=str(v))
    elif isinstance(v, (list, tuple)):
        choices = tuple((k, k) for k in v)
        return forms.MultipleChoiceField(choices=choices)
    elif isinstance(v, OrderedDict):
        choices = tuple((k, subv) for k, subv in v.items())
        return forms.ChoiceField(choices=choices)
    elif isinstance(v, dict):
        choices = sorted((k, subv) for k, subv in v.items())
        return forms.ChoiceField(choices=choices)
    elif isinstance(v, (set, frozenset)):
        choices = tuple((k, k) for k in v)
        return forms.ChoiceField(choices=choices)
    elif isinstance(v, Model):
        kws = {'queryset': v.__class__.objects.all()}
        if hasattr(v, 'pk') and v.pk is not None:
            kws.update(initial=v.pk)
        return forms.ModelChoiceField(**kws)
    elif isinstance(v, QuerySet):
        return forms.ModelMultipleChoiceField(queryset=v)
    elif isinstance(v, string_types):
        try:
            validate_ipv46_address(v)
            return forms.GenericIPAddressField(initial=v)
        except ValidationError:
            pass
        try:
            URLValidator()(v)
            return forms.URLField(initial=v)
        except ValidationError:
            pass
        try:
            validate_email(v)
            return forms.EmailField(initial=v)
        except ValidationError:
            pass
        if ' ' not in v and '-' in v:
            try:
                validate_slug(v)
                return forms.SlugField(initial=v)
            except ValidationError:
                pass
        return forms.CharField(initial=v)


def generate_form(dictionary):
    form_fields = OrderedDict()
    for k, v in sorted(dictionary.items()):
        if callable(v):
            v = v()
        form_fields[k] = _select_field(v)
    if len(form_fields) != len(dictionary):
        raise ValidationError("Could not generate all fields for form")
    return type(str('DictionaryGeneratedForm'), (forms.Form,), form_fields)











# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
import json
from uuid import UUID, uuid4
from datetime import timedelta, datetime, date, time
from decimal import Decimal
from django.contrib.auth import get_user_model
from django.test import TransactionTestCase
from django.test.utils import override_settings
from django.utils.timezone import utc
import pytest
from stagesetting.models import RuntimeSetting
from stagesetting.utils import JSONEncoder, FormRegistry, generate_form


def test_custom_json():
    output = json.dumps({
        'uuid': UUID('98967ef2-a5a3-4c19-aefa-9bb8dc5fcbac'),
        'timedelta': timedelta(minutes=14),
        'datetime': datetime(2015, 8, 1, 16, 8, 51, 125068),
        'tzdatetime': datetime(2015, 8, 1, 16, 8, 51, 125068).replace(tzinfo=utc),
        'super': 1,
    }, cls=JSONEncoder)
    assert '"timedelta": "840.0"' in output
    assert '"datetime": "2015-08-01 16:08:51.125068"' in output
    assert '"uuid": "98967ef2-a5a3-4c19-aefa-9bb8dc5fcbac"' in output
    assert '"tzdatetime": "2015-08-01 16:08:51.125068"' in output


def formregistry_ready():
    fr = FormRegistry(name='default')
    newconfig = {
        'HELLO': ['test_app.forms.DatetimeForm', {'blip': 'blop'}],
        'HELLO2': ['test_app.forms.ListPerPageForm'],
    }
    with override_settings(STAGESETTINGS=newconfig):
        return fr.ready(sender=None, instance=None, model=RuntimeSetting)


@pytest.mark.django_db
def test_formregistry_ready():
    result = formregistry_ready()
    assert len(result) == 2


class FormRegistryTestCase(TransactionTestCase):
    def test_ready(self):
        with self.assertNumQueries(3):
            result = formregistry_ready()
        self.assertEqual(len(result), 2)


@pytest.mark.django_db
def test_generate_form():
    def user():
        return get_user_model()()

    def user_qs():
        return get_user_model().objects.all()

    user = get_user_model().objects.create()

    data = {
        'int': 1,
        'email': 'a@b.com',
        'url': 'https://news.bbc.co.uk/',
        'model': user,
        'queryset': user_qs,
        'datetime': datetime.today(),
        'date': date.today(),
        'time': time(4, 23),
        'boolean': False,
        'nullboolean': None,
        'ip': '127.0.0.1',
        'slug': 'test-test',
        'text': 'char field',
        'decimal': Decimal('3.25'),
        'float': 2.3,
        'uuid': uuid4(),
        'list': ['a', 'b'],
        'set': {'a', 'b'}
    }
    form_class = generate_form(data)
    uuid = uuid4()
    dt = datetime.today()
    daaate = dt.date()
    validate_data = {
        'int': 1,
        'email': 'a@b.com',
        'url': 'https://news.bbc.co.uk/',
        'model': str(user.pk),
        'queryset': [str(user.pk)],
        'datetime': dt.strftime('%Y-%m-%d %H:%M:%S'),
        'date': daaate.strftime('%Y-%m-%d'),
        'time': '4:23',
        'boolean': '',
        'nullboolean': '0',
        'ip': '127.0.0.1',
        'slug': 'test-test',
        'text': 'char field',
        'decimal': '3.25',
        'float': '2.3',
        'uuid': str(uuid),
        'list': ['a'],
        'set': 'b'
    }
    form = form_class(data=validate_data)
    valid = form.is_valid()
    assert form.errors == {}
    qs = form.cleaned_data.pop('queryset')
    uuid_result = form.cleaned_data.pop('uuid')
    assert form.cleaned_data == {
        'url': 'https://news.bbc.co.uk/',
        'text': 'char field',
        'nullboolean': None,
        'datetime': dt.replace(microsecond=0),
        'decimal': Decimal('3.25'),
        'boolean': False,
        'ip': '127.0.0.1',
        'float': 2.3,
        'email': 'a@b.com',
        'slug': 'test-test',
        'model': user,
        'int': 1,
        'list': ['a'],
        'time': time(4, 23),
        'set': 'b',
        'date': daaate
    }
    assert valid is True



from __future__ import unicode_literals
from __future__ import absolute_import
import json
from threading import RLock
from django.core.cache.backends.base import MEMCACHE_MAX_KEY_LENGTH
from django.core.exceptions import ValidationError
from django.db.models.manager import Manager
from django.db.models.query import QuerySet
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django.db.models import Model, TextField
from django.db.models.fields import CharField
from django.db.models.fields import DateTimeField
from .utils import registry
from .validators import validate_setting_name


class RuntimeSettingQuerySet(QuerySet):
    def key_exists(self, key):
        try:
            validate_setting_name(key)
        except ValidationError as e:
            raise self.model.DoesNotExist("Invalid setting name")
        return self.filter(key=key).exists()


@python_2_unicode_compatible
class RuntimeSetting(Model):
    key = CharField(max_length=MEMCACHE_MAX_KEY_LENGTH, unique=True,
                    db_index=True, validators=[validate_setting_name],
                    verbose_name=_("Name"))
    raw_value = TextField()
    created = DateTimeField(auto_now_add=True)
    modified = DateTimeField(auto_now=True)

    objects = Manager.from_queryset(RuntimeSettingQuerySet)()

    def __str__(self):
        return self.key

    def get_form_class(self):
        return registry[self.key]

    def get_form(self):
        data = registry.deserialize(self.raw_value)
        return self.get_form_class()(data=data, initial=data, files=None)

    def get_value(self):
        form = self.get_form()
        form.full_clean()
        return form.cleaned_data

    def set_value(self, value):
        self.raw_value = registry.serialize(value)

    value = property(get_value, set_value)

    @property
    def default_value(self):
        return registry._get_default(key=self.key)

    def has_changed(self):
        return self.value != self.default_value
    has_changed.short_description = _("Changed")
    has_changed.boolean = True

    def __repr__(self):
        return '<%(cls)s key="%(key)s", raw_value="%(value)s">' % {
            'mod': self.__module__, 'cls': self.__class__.__name__,
            'key': self.key, 'value': self.raw_value,
        }

    def delete(self, using=None):
        assert self._get_pk_val() is not None, (
            "%s object can't be deleted because its %s attribute is set to None." %
            (self._meta.object_name, self._meta.pk.attname)
        )
        default = registry.get_default(self.key)
        self.raw_value = default
        self.full_clean()
        self.save()
    delete.alters_data = True

    class Meta:
        ordering = ('key',)
        verbose_name = _("Setting")
        verbose_name_plural = _("Settings")


@python_2_unicode_compatible
class RuntimeSettingWrapper(object):
    __slots__ = ('settings', '_lock')
    def __init__(self, settings=None):
        super(RuntimeSettingWrapper, self).__setattr__('settings', settings)
        super(RuntimeSettingWrapper, self).__setattr__('_lock', RLock())

    def __str__(self):
        msg_dict = {'cls': self.__class__.__name__}
        if self.settings is None:
            return 'Unevaluated %(cls)s' % msg_dict
        return 'Evaluated %(cls)s' % msg_dict

    def __repr__(self):
        msg_dict = {'cls': self.__class__.__name__}
        if self.settings is None:
            msg_dict.update(settings=self.settings)
            return '<%(cls)s [Unevaluated] settings=%(settings)r>' % msg_dict
        msg_dict.update(settings=self.settings.keys())
        return '<%(cls)s [Evaluated] settings=%(settings)r>' % msg_dict

    def __dir__(self):
        exposed = ['settings']
        if self.settings is None:
            return exposed
        return self.settings.keys() + exposed

    def _fetch_settings(self):
        if self.settings is not None:
            return False

        with self._lock:
            settings = {}
            # Set up defaults which are registered.
            for default_key, default_value in registry._defaults.items():
                settings[default_key] = default_value

            # Set up anything that's been configured into the database.
            keys = frozenset(registry._registry.keys())
            for setting in RuntimeSetting.objects.filter(key__in=keys).iterator():  # noqa
                try:
                    settings[setting.key] = setting.value
                except ValidationError:
                    continue

            super(RuntimeSettingWrapper, self).__setattr__('settings', settings)
        return True

    def __getitem__(self, item):
        self._fetch_settings()
        return self.settings[item]

    def __getattr__(self, item):
        self._fetch_settings()
        return self.settings[item]

    def __len__(self):
        self._fetch_settings()
        return len(self.settings)

    def __contains__(self, item):
        self._fetch_settings()
        return item in self.settings

    def __iter__(self):
        self._fetch_settings()
        return self.settings.__iter__()

    def __setattr__(self, key, value):
        raise NotImplementedError

    def __delattr__(self, key):
        raise NotImplementedError

    def __setitem__(self, item, value):
        raise NotImplementedError

    def __delitem__(self, item):
        raise NotImplementedError



#! /usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from datetime import datetime, date, time
import os
import sys
from django.conf import settings

DEBUG = os.environ.get('DEBUG', 'on') == 'on'
SECRET_KEY = os.environ.get('SECRET_KEY', 'TESTTESTTESTTESTTESTTESTTESTTEST')
ALLOWED_HOSTS = os.environ.get('ALLOWED_HOSTS', 'localhost,testserver').split(',')

BASE_DIR = os.path.abspath(os.path.dirname(os.path.abspath(__file__)))

def user():
    from django.contrib.auth import get_user_model
    return get_user_model()()

def user_qs():
    from django.contrib.auth import get_user_model
    return get_user_model().objects.all()

settings.configure(
    DEBUG=DEBUG,
    SECRET_KEY=SECRET_KEY,
    ALLOWED_HOSTS=ALLOWED_HOSTS,
    SITE_ID=1,
    ROOT_URLCONF='test_urls',  # or __name__ to use local ones ...
    MIDDLEWARE_CLASSES=(
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'stagesetting.middleware.ApplyRuntimeSettings',
    ),
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
        }
    },
    TEMPLATE_CONTEXT_PROCESSORS=(
        'django.contrib.messages.context_processors.messages',
        'django.contrib.auth.context_processors.auth',
        'stagesetting.context_processors.runtime_settings',
    ),
    INSTALLED_APPS=(
        'django.contrib.contenttypes',
        'django.contrib.messages',
        'django.contrib.sites',
        'django.contrib.auth',
        'django.contrib.staticfiles',
        'django.contrib.admin',
        'stagesetting',
        'test_app',
        'debug_toolbar',
    ),
    STATIC_ROOT=os.path.join(BASE_DIR, 'test_collectstatic'),
    STATIC_URL='/__static__/',
    MESSAGE_STORAGE='django.contrib.messages.storage.cookie.CookieStorage',
    SESSION_ENGINE='django.contrib.sessions.backends.signed_cookies',
    SESSION_COOKIE_HTTPONLY=True,
    STAGESETTINGS={
        'LIST_PER_PAGE': ['test_app.forms.ListPerPageForm'],
        'DATES': ['test_app.forms.DateForm', {'start': date.today()}],
        'DATETIMES': ['test_app.forms.DatetimeForm'],
        'USERS': ['test_app.forms.ModelChoicesForm'],
        'GENERATED':  {
            'int': 1,
            'email': 'a@b.com',
            'url': 'https://news.bbc.co.uk/',
            'model': user,
            'queryset': user_qs,
            'datetime': datetime.today(),
            'date': date.today(),
            'time': time(4, 23),
            'boolean': False,
            'nullboolean': None,
            'ip': '127.0.0.1',
            'slug': 'test-test',
            'text': 'char field',
        }
    },
)

from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()


if __name__ == "__main__":
    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.conf.urls import url
from .views import create_view
from .views import update_view
from .views import delete_view
from .views import list_view

stagesetting_create = url(regex=r'^add/$',
                     view=create_view,
                     name='stagesetting_create',
                     kwargs={})

stagesetting_update = url(regex=r'^(?P<pk>\d+)/update/$',
                     view=update_view,
                     name='stagesetting_update',
                     kwargs={})

stagesetting_delete = url(regex=r'^(?P<pk>\d+)/delete/$',
                     view=delete_view,
                     name='stagesetting_delete',
                     kwargs={})

stagesetting_list = url(regex=r'^$',
                    view=list_view,
                    name='stagesetting_list',
                    kwargs={})

urlpatterns = [
    stagesetting_create,
    stagesetting_update,
    stagesetting_delete,
    stagesetting_list,
]



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.utils.translation import ugettext_lazy as _
from re import compile


setting_name_re_str = '^[A-Z][A-Z0-9_]+[A-Z0-9]$'

class SettingNameValidator(RegexValidator):
    message = _("Setting format should be CAPITAL_WITH_UNDERSCORES")
    regex = compile(setting_name_re_str)

validate_setting_name = SettingNameValidator()


def validate_formish(value):
    try:
        assert hasattr(value, 'is_valid')
        assert hasattr(value, 'clean')
    except AssertionError:
        raise ValidationError("%(form)r doesn't appear to be a Form class" % {
            'form': value})


def validate_default(value):
    try:
        assert hasattr(value, '__getitem__')
        assert hasattr(value, 'keys')
    except AssertionError:
        raise ValidationError("%r doesn't appear to be dictish" % value)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from .models import RuntimeSettingWrapper


def runtime_settings(request):
    if hasattr(request, 'stagesetting'):
        settings = request.stagesetting
    else:
        settings = RuntimeSettingWrapper()
    return {
        'STAGESETTING': settings,
    }



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import stagesetting.validators


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RuntimeSetting',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('key', models.CharField(db_index=True, unique=True, max_length=250, verbose_name='Name', validators=[stagesetting.validators.SettingNameValidator()])),
                ('raw_value', models.TextField()),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ('key',),
                'verbose_name': 'Setting',
                'verbose_name_plural': 'Settings',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
import django
from django.conf import settings
import os


HERE = os.path.realpath(os.path.dirname(__file__))


def pytest_configure():
    if not settings.configured:
        settings.configure(
            DATABASES={
                "default": {
                    "ENGINE": "django.db.backends.sqlite3",
                    "NAME": ":memory:"
                    }
                },
            INSTALLED_APPS=(
                'django.contrib.sites',
                'django.contrib.sitemaps',
                'django.contrib.auth',
                'django.contrib.admin',
                'django.contrib.sessions',
                'django.contrib.contenttypes',
                'django.contrib.messages',
                'stagesetting',
            ),
            # these are the default in 1.8, so we should make sure we
            # work with those.
            MIDDLEWARE_CLASSES=(
                'django.contrib.sessions.middleware.SessionMiddleware',
                'django.middleware.common.CommonMiddleware',
                'django.middleware.csrf.CsrfViewMiddleware',
                'django.contrib.auth.middleware.AuthenticationMiddleware',
                # 'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
                'django.contrib.messages.middleware.MessageMiddleware',
                'django.middleware.clickjacking.XFrameOptionsMiddleware',
            ),
            BASE_DIR=HERE,
            SITE_ID=1,
            STATIC_URL='/__s__/',
            STATIC_ROOT=os.path.join(HERE, 'test_collectstatic'),
            ROOT_URLCONF='test_urls',
            PASSWORD_HASHERS=(
                'django.contrib.auth.hashers.MD5PasswordHasher',
            ),
        )
    if hasattr(django, 'setup'):
        django.setup()



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.utils.translation import ugettext_lazy as _
from django.apps import AppConfig


class TestAppConfig(AppConfig):
    name = 'test_app'
    verbose_name = _("Test app")




# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.forms import Form, DateTimeField, IntegerField, ModelChoiceField, \
    ModelMultipleChoiceField
from django.forms import DateField


class DateForm(Form):
    start = DateField()
    end = DateField()

    def clean(self):
        cd = self.cleaned_data
        if 'start' in cd and 'end' in cd and cd['start'] > cd['end']:
            raise ValidationError("nope")
        return cd


class DatetimeForm(Form):
    start = DateTimeField()
    end = DateTimeField()

    def clean(self):
        cd = self.cleaned_data
        if 'start' in cd and 'end' in cd and cd['start'] > cd['end']:
            # removing from cleaned_data is a complete failboat, so screw that.
            self._errors["end"] = self.error_class(["End must be after start"])
        return cd


class ListPerPageForm(Form):
    count = IntegerField(initial=25, min_value=1, max_value=99)


class ModelChoicesForm(Form):
    single_user = ModelChoiceField(queryset=get_user_model().objects.all())
    many_users = ModelMultipleChoiceField(queryset=get_user_model().objects.all())
    another = IntegerField(min_value=1, max_value=5)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
import logging
from django.core.checks import registry as django_check_registry
from django.db.utils import OperationalError
from django.utils.translation import ugettext_lazy as _
from django.apps import AppConfig


logger = logging.getLogger(__name__)


class StageSettingAppConfig(AppConfig):
    name = 'stagesetting'
    verbose_name = _("Run configuration")

    def ready(self):
        from .models import RuntimeSetting
        from .checks import check_setting
        from .utils import registry as stagesetting_registry
        django_check_registry.register(check_setting)
        # So I want to patch the defaults in by default, but apparently I
        # can't do so without migrating, nor in post_migrate.
        # I don't yet know in what ways this will break.
        try:
            stagesetting_registry.ready(sender=self.__class__, instance=self,
                                        model=RuntimeSetting)
        except OperationalError as exc:
            logger.warning("Could not do the database work in AppConfig.ready, "
                           "probably because nothing has migrated yet",
                           exc_info=1)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.conf.urls import url, include
from django.contrib import admin
from stagesetting import urls as stagesetting_urls

urlpatterns = [
   url(r'^', include(stagesetting_urls)),
   url(r'^test_admin/', include(admin.site.urls)),
]
